<?php

namespace QuadLayers\WOOCCM_PRO;

use QuadLayers\WOOCCM_PRO\Compatibility\Fields_Invoices;
use QuadLayers\WOOCCM_PRO\Compatibility\Fields_Subscriptions;

use QuadLayers\WOOCCM_PRO\View\Backend\License;
use QuadLayers\WOOCCM_PRO\View\Backend\Order; // is a controller
use QuadLayers\WOOCCM_PRO\View\Backend\Fields_Export;

use QuadLayers\WOOCCM_PRO\View\Frontend\Email; // is a controller
use QuadLayers\WOOCCM_PRO\View\Frontend\MyAccount; // is a controller
use QuadLayers\WOOCCM_PRO\View\Frontend\Fields_Price;
use QuadLayers\WOOCCM_PRO\View\Frontend\Fields_Display;
use QuadLayers\WOOCCM_PRO\View\Frontend\Fields_Select2;
use QuadLayers\WOOCCM_PRO\View\Frontend\Fields_Timepicker;
use QuadLayers\WOOCCM_PRO\View\Frontend\Fields_Datepicker;
use QuadLayers\WOOCCM_PRO\View\Frontend\Fields_Filters;
use QuadLayers\WOOCCM_PRO\View\Frontend\Fields_Validate_Regex;

/**
 *Plugin Class
 */
final class Plugin {

	protected static $_instance;

	private function __construct() {
		/**
		 * Load plugin textdomain.
		 */
		add_action( 'init', array( $this, 'load_textdomain' ) );

		add_action( 'init', array( $this, 'compatibility' ) );

		add_action(
			'wooccm_init',
			function () {

				/**
				 * Remove premium CSS
				 */
				remove_action( 'admin_footer', array( 'QuadLayers\\WOOCCM\\Plugin', 'add_premium_css' ) );
				/**
				 * Remove premium menu
				 */
				remove_action( 'admin_menu', array( 'QuadLayers\WOOCCM\Controller\Premium', 'add_menu' ) );
				remove_filter( 'wooccm_sections_header', array( 'QuadLayers\WOOCCM\Controller\Premium', 'add_header' ) );
				/**
				 * Load classes
				 */
				License::instance();
				Order::instance();
				Fields_Export::instance();
				Email::instance();

				if ( ! is_admin() ) {
					MyAccount::instance();
					Fields_Price::instance();
					Fields_Display::instance();
					Fields_Select2::instance();
					Fields_Timepicker::instance();
					Fields_Datepicker::instance();
					Fields_Filters::instance();
					Fields_Validate_Regex::instance();
				}
			}
		);

		add_filter( 'wooccm_fields_disabled_types', '__return_empty_array' );
	}

	public function load_textdomain() {
		load_plugin_textdomain( 'woocommerce-checkout-manager-pro', false, WOOCCM_PRO_PLUGIN_DIR . '/languages/' );
	}

	public function compatibility() {
		Fields_Invoices::instance();
		Fields_Subscriptions::instance();
	}

	public function register_scripts() {
		// UI
		// ---------------------------------------------------------------------
		wp_register_style( 'jquery-ui-style', WC()->plugin_url() . '/assets/css/jquery-ui/jquery-ui.min.css', array(), WC_VERSION );

		// Timepicker
		// ---------------------------------------------------------------------
		wp_register_style( 'jquery-ui-timepicker', plugins_url( 'assets/frontend/timepicker/jquery.ui.timepicker.css', WOOCCM_PRO_PLUGIN_FILE ), array( 'jquery-ui-style' ), WOOCCM_PRO_PLUGIN_VERSION );
		wp_register_script( 'jquery-ui-timepicker', plugins_url( 'assets/frontend/timepicker/jquery.ui.timepicker.js', WOOCCM_PRO_PLUGIN_FILE ), array( 'jquery' ), WOOCCM_PRO_PLUGIN_VERSION, true );
	}

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}
}

function WOOCCM_PRO() {     // phpcs:ignore WordPress.NamingConventions.ValidFunctionName.FunctionNameInvalid
	return Plugin::instance();
}

WOOCCM_PRO();
