<?php

namespace QuadLayers\WOOCCM_PRO\Compatibility;

use QuadLayers\WOOCCM\Plugin;

/**
 * Fields_Invoices Class
 */
class Fields_Invoices {

	protected static $_instance;

	public function __construct() {

		/*
		add_filter( 'wooccm_pro_invoice_show_fields', function( $show, $order, $type = null ) {
			$show = false;
			return $show;
		}, 10, 3 );
		 */
		// Print Invoice & Delivery Notes for WooCommerce
		// https://wordpress.org/plugins/woocommerce-delivery-notes/
		add_filter( 'wcdn_order_info_fields', array( $this, 'wcdn_order_info_fields' ), 10, 2 );
		// PDF Invoice
		// https://docs.woocommerce.com/documentation/plugins/woocommerce/woocommerce-extensions/pdf-invoice/
		add_filter( 'pdf_content_additional_content', array( $this, 'pdf_content_additional_content' ), 10, 2 );
		// WooCommerce PDF Invoices & Packing Slips
		// https://wordpress.org/plugins/woocommerce-pdf-invoices-packing-slips/
		add_action( 'wpo_wcpdf_after_order_details', array( $this, 'wpo_wcpdf_after_order_details' ), 10, 2 );
	}

	public static function instance() {

		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	protected function get_invoice_fields( $order ) {

		if ( ! function_exists( 'WOOCCM' ) ) {
			return;
		}

		$invoice_billing_fields    = array();
		$invoice_shipping_fields   = array();
		$invoice_additional_fields = array();

		$billing_fields   = Plugin::instance()->billing->get_fields();
		$billing_defaults = array_column( Plugin::instance()->billing->get_defaults(), 'key' );

		$shipping_fields   = Plugin::instance()->shipping->get_fields();
		$shipping_defaults = array_column( Plugin::instance()->shipping->get_defaults(), 'key' );

		$additional_fields = Plugin::instance()->additional->get_fields();

		foreach ( $billing_fields as $field_id => $field ) {
			if ( ! in_array( $field['key'], $billing_defaults ) && empty( $field['hide_invoice'] ) ) {
				$value = $order->get_meta( sprintf( '_%s', $field['key'] ), true );
				if ( $value ) {
					$invoice_billing_fields[ $field['key'] ] = array(
						'key'   => $field['key'],
						'label' => $field['label'],
						'value' => $value,
					);
				}
			}
		}

		foreach ( $shipping_fields as $field_id => $field ) {
			if ( ! in_array( $field['key'], $shipping_defaults ) && empty( $field['hide_invoice'] ) ) {
				$value = $order->get_meta( sprintf( '_%s', $field['key'] ), true );
				if ( $value ) {
					$invoice_shipping_fields[ $field['key'] ] = array(
						'key'   => $field['key'],
						'label' => $field['label'],
						'value' => $value,
					);
				}
			}
		}

		foreach ( $additional_fields as $field_id => $field ) {
			if ( empty( $field['hide_invoice'] ) ) {
				$value = $order->get_meta( sprintf( '_%s', $field['key'] ), true );
				if ( $value ) {
					$invoice_additional_fields[ $field['key'] ] = array(
						'key'   => $field['key'],
						'label' => $field['label'],
						'value' => $value,
					);
				}
			}
		}
		return array(
			$invoice_billing_fields,
			$invoice_shipping_fields,
			$invoice_additional_fields,
		);
	}

	public function wcdn_order_info_fields( $fields, $order ) {

		if ( ! apply_filters( 'wooccm_pro_invoice_show_fields', '__return_true', $order, null ) ) {
			return $fields;
		}

		/**
		 * Refund is treated as order child, and this object does not have the get_order_number method.
		 */
		if ( method_exists( $order, 'get_parent_id' ) && $order->get_parent_id() ) {
			$order = wc_get_order( $order->get_parent_id() );
		}

		list( $invoice_billing_fields, $invoice_shipping_fields, $invoice_additional_fields ) = $this->get_invoice_fields( $order );

		if (
		! empty( $invoice_billing_fields )
		||
		! empty( $invoice_shipping_fields )
		||
		! empty( $invoice_additional_fields )
		) {
			return array_merge(
				$fields,
				$invoice_billing_fields,
				$invoice_shipping_fields,
				$invoice_additional_fields
			);
		}

		return $fields;
	}


	public function wpo_wcpdf_after_order_details( $type, $order ) {

		if ( ! apply_filters( 'wooccm_pro_invoice_show_fields', '__return_true', $order, $type ) ) {
			return;
		}

		/**
		 * Refund is treated as order child, and this object does not have the get_order_number method.
		 */
		if ( method_exists( $order, 'get_parent_id' ) && $order->get_parent_id() ) {
			$order = wc_get_order( $order->get_parent_id() );
		}

		list( $invoice_billing_fields, $invoice_shipping_fields, $invoice_additional_fields ) = $this->get_invoice_fields( $order );

		if (
		! empty( $invoice_billing_fields )
		||
		! empty( $invoice_shipping_fields )
		||
		! empty( $invoice_additional_fields )
		) {

			$fields = array_merge(
				$invoice_billing_fields,
				$invoice_shipping_fields,
				$invoice_additional_fields
			);

			$title = get_option( 'wooccm_invoice_custom_fields_title', __( 'Order Additional', 'woocommerce-checkout-manager-pro' ) );
			?>
			<h2><?php echo esc_html( $title ); ?></h2>

			<table style="width: 100%;">
				<thead>
				<tr>
					<th class="product"><?php esc_html_e( 'Label', 'woocommerce-checkout-manager-pro' ); ?></th>
					<th class="quantity"><?php esc_html_e( 'Value', 'woocommerce-checkout-manager-pro' ); ?></th>
				</tr>
				</thead>
				<tbody>
					<?php foreach ( $fields as $key => $field ) : ?>
						<tr id="tr-<?php echo esc_attr( $field['key'] ); ?>">
							<td class="product">
								<span class="item-name">
								<?php echo esc_html( $field['label'] ); ?>
								</span>
							</td>
							<td class="price">
								<?php echo esc_html( $field['value'] ); ?>
							</td>
						</tr>
					<?php endforeach; ?>
				</tbody>
			</table>
			<?php

		}
	}

	public function pdf_content_additional_content( $output, $order_id ) {

		$order = wc_get_order( $order_id );
		if ( ! apply_filters( 'wooccm_pro_invoice_show_fields', '__return_true', $order, null ) ) {
			return $output;
		}

		list( $invoice_billing_fields, $invoice_shipping_fields, $invoice_additional_fields ) = $this->get_invoice_fields( $order );

		if (
		! empty( $invoice_billing_fields )
		||
		! empty( $invoice_shipping_fields )
		||
		! empty( $invoice_additional_fields )
		) {

			$fields = array_merge(
				$invoice_billing_fields,
				$invoice_shipping_fields,
				$invoice_additional_fields
			);

			ob_start();
			$title = get_option( 'wooccm_invoice_custom_fields_title', __( 'Order Additional', 'woocommerce-checkout-manager-pro' ) );
			?>
			<h2><?php echo esc_html( $title ); ?></h2>
			<table class="shop_table orderdetails" width="100%">
				<thead>
				<tr class="pdf_table_row">
					<th class="pdf_table_cell" width="5%" align="right" valign="top"><?php esc_html_e( 'Label', 'woocommerce-checkout-manager-pro' ); ?></th>
					<th class="pdf_table_cell" width="95%" align="right" valign="top"><?php esc_html_e( 'Value', 'woocommerce-checkout-manager-pro' ); ?></th>
				</tr>
				</thead>
				<tbody>
					<?php foreach ( $fields as $key => $field ) : ?>
						<tr>
							<td valign="top">
								<?php echo esc_html( $field['label'] ); ?>
							</td>
							<td align="right">
								<?php echo esc_html( $field['value'] ); ?>
							</td>
						</tr>
					<?php endforeach; ?>
				</tbody>
			</table>
			<?php
			return $output . ob_get_clean();
		}
		return $output;
	}
}
