<?php

namespace QuadLayers\WOOCCM_PRO\View\Backend;

use QuadLayers\WOOCCM\Plugin;

/**
 * Order Class
 */
class Order {

	protected static $_instance;

	public function __construct() {
		add_action( 'wp_ajax_wooccm_search_field', array( $this, 'ajax_search_field' ) );
		add_action( 'woocommerce_admin_order_data_after_shipping_address', array( $this, 'add_order_data' ) );

		// List Legacy
		// -------------------------------------------------------------------------

		add_filter( 'manage_edit-shop_order_columns', array( $this, 'render_column_titles' ), 15 );
		add_action( 'manage_shop_order_posts_custom_column', array( $this, 'render_column_content' ), 5, 2 );
		add_filter( 'manage_edit-shop_order_sortable_columns', array( $this, 'add_sortable_columns' ) );
		add_action( 'restrict_manage_posts', array( $this, 'restrict_orders' ), 15 );

		// List HPOS
		// -------------------------------------------------------------------------
		add_filter( 'woocommerce_shop_order_list_table_columns', array( $this, 'render_column_titles' ), 15 );
		add_action( 'woocommerce_shop_order_list_table_custom_column', array( $this, 'render_column_content' ), 5, 2 );
		add_filter( 'woocommerce_shop_order_list_table_sortable_columns', array( $this, 'add_sortable_columns' ) );
		add_action( 'woocommerce_shop_order_list_table_prepare_items_query_args', array( $this, 'sortable_columns_prepare_items_query_args' ), 15, 1 );
		add_action( 'woocommerce_order_list_table_restrict_manage_orders', array( $this, 'restrict_orders' ), 15, 1 );

		// Filter
		// -------------------------------------------------------------------------
		add_action( 'parse_query', array( $this, 'search_custom_fields' ) );

		// Filter HPOS
		// -------------------------------------------------------------------------
		add_filter( 'woocommerce_order_list_table_prepare_items_query_args', array( $this, 'search_custom_fields_hpos' ) );

		// Save
		// -------------------------------------------------------------------------

		add_action( 'woocommerce_process_shop_order_meta', array( $this, 'save_order_data' ) );
	}

	public function add_order_data() {
		?>
		<style>
			#order_data .load_customer_additional {
				display:none!important;
			}
		</style>
		<?php
	}

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function ajax_search_field() {
		global $wpdb;

		if ( check_admin_referer( 'wooccm_search_field', 'nonce' ) && isset( $_GET['term'] ) && isset( $_GET['key'] ) ) {

			$term         = sanitize_text_field( wp_unslash( $_GET['term'] ) );
			$meta_key     = '_' . strip_tags( wc_clean( wp_unslash( $_GET['key'] ) ) );
			$field_values = array();

			$orders = wc_get_orders( array( $meta_key => $term ) );

			foreach ( $orders as $order_id ) {
				$order = wc_get_order( $order_id );
				if ( $order ) {
					// Assuming the meta data you are looking for is stored in a consistent format
					// Adjust the logic here if you're looking for multiple or complex meta data values
					$meta_value = '';
					if ( method_exists( $order, "get$meta_key" ) ) {
						$meta_value = $order->{"get$meta_key"}();
					} else {
						$meta_value = $order->get_meta( $meta_key, true );
					}
					if ( ! empty( $meta_value ) && str_contains( $meta_value, $term ) ) {
						$field_values[ $meta_value ] = $meta_value;
					}
				}
			}

			wp_send_json( $field_values );
		}

		wp_die();
	}

	// List
	// ---------------------------------------------------------------------------

	protected function restrict_orders_field( $field ) {

		$option = Plugin::instance()->billing->get_option_types();

		if ( count( $field['options'] ) && in_array( $field['type'], $option ) ) {

			$value = isset( $_GET[ $field['key'] ] ) ? wc_clean( wp_unslash( $_GET[ $field['key'] ] ) ) : null;
			?>
		<select class="wc-enhanced-select" name="<?php echo esc_attr( $field['key'] ); ?>" data-placeholder="<?php printf( esc_attr( __( 'Show all %s', 'woocommerce-checkout-manager-pro' ) ), esc_attr( $field['label'] ) ); ?>" data-allow_clear="true" style="min-width:200px;">
		<option value=""></option>
			<?php foreach ( $field['options'] as $option ) : ?>
				<?php if ( ! empty( $option['label'] ) ) { ?>
			<option value="<?php echo esc_html( $option['label'] ); ?>" <?php selected( $option['label'], $value ); ?>><?php echo esc_html( $option['label'] ); ?></option>
					<?php
				}
			endforeach;
			?>
		</select>
			<?php
		} elseif ( 'checkbox' === $field['type'] || 'file' === $field['type'] ) {
			?>
		<label style="line-height: 32px; margin: 0 10px;">
		<input style="height:1rem" type="checkbox" id="<?php echo esc_attr( $field['key'] ); ?>" name="<?php echo esc_attr( $field['key'] ); ?>" value="1" <?php checked( isset( $_GET[ $field['key'] ] ) && wp_unslash( $_GET[ $field['key'] ] ), true, true ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized ?> />
			<?php printf( '%s', $field['label'] ? esc_html( $field['label'] ) : sprintf( esc_html__( 'Field %s', 'woocommerce-checkout-manager-pro' ), $field['id'] ) ); // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped ?>
		</label>
			<?php
		} else /*if ('text' === $field['type'])*/ {
			// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			$key = isset( $_GET[ $field['key'] ] ) ? wp_unslash( $_GET[ $field['key'] ] ) : null;
			?>
		<select class="wooccm-enhanced-search" name="<?php echo esc_attr( $field['key'] ); ?>" data-placeholder="<?php printf( esc_attr__( 'Show all %s', 'woocommerce-checkout-manager-pro' ), esc_html( $field['label'] ) ); ?>" data-allow_clear="true" data-key="<?php echo esc_attr( $field['key'] ); ?>" style="min-width:200px;">
			<?php if ( ! empty( $key ) ) : ?>
			<option value="<?php echo esc_attr( $key ); ?>" selected="selected"><?php echo esc_html( $key ); ?></option>
		<?php endif; ?>
		</select>
			<?php
		}
	}

	public function restrict_orders( $typenow ) {

		if ( 'shop_order' !== $typenow ) {
			return;
		}
		$fields = Plugin::instance()->billing->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['filterable'] ) ) {
					$this->restrict_orders_field( $field );
				}
			}
		}
		$fields = Plugin::instance()->shipping->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['filterable'] ) ) {
					$this->restrict_orders_field( $field );
				}
			}
		}
		$fields = Plugin::instance()->additional->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['filterable'] ) ) {
					$this->restrict_orders_field( $field );
				}
			}
		}
	}

	public function render_column_titles( $columns ) {

		// get all columns up to and excluding the 'order_actions' column
		$new_columns = array();

		foreach ( $columns as $name => $value ) {

			if ( 'order_actions' === $name ) {
				prev( $columns );
				break;
			}

			$new_columns[ $name ] = $value;
		}

		$fields = Plugin::instance()->billing->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['listable'] ) ) {
					$new_columns[ $field['key'] ] = sprintf( '%s', $field['label'] ? esc_html( $field['label'] ) : sprintf( esc_html__( 'Field %s', 'woocommerce-checkout-manager-pro' ), $field_id ) );
				}
			}
		}
		$fields = Plugin::instance()->shipping->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['listable'] ) ) {
					$new_columns[ $field['key'] ] = sprintf( '%s', $field['label'] ? esc_html( $field['label'] ) : sprintf( esc_html__( 'Field %s', 'woocommerce-checkout-manager-pro' ), $field_id ) );
				}
			}
		}
		$fields = Plugin::instance()->additional->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['listable'] ) ) {
					$new_columns[ $field['key'] ] = sprintf( '%s', $field['label'] ? esc_html( $field['label'] ) : sprintf( esc_html__( 'Field %s', 'woocommerce-checkout-manager-pro' ), $field_id ) );
				}
			}
		}
		// add the 'order_actions' column, and any others
		foreach ( $columns as $name => $value ) {
			$new_columns[ $name ] = $value;
		}

		return $new_columns;
	}

	protected function render_column_content_field( $order, $field ) {

		switch ( $field['type'] ) {

			case 'checkbox':
				echo $order->get_meta( sprintf( '_%s', $field['key'] ), true ) ? '&#x2713;' : '';
				break;

			case 'file':
				$file_ids = explode( ',', $order->get_meta( sprintf( '_%s', $field['key'] ), true ) );

				if ( ! empty( $file_ids[0] ) ) {

					echo '<ul class="wooccm-files" style="list-style:none">';
					foreach ( $file_ids as $key => $file_id ) {
						$url = get_edit_post_link( $file_id );
						if ( $url ) {

							$image = wp_get_attachment_image_src( $file_id );

							echo '<li style="display:inline-block"><a style="display:block" href="' . esc_url( $url ) . '"><img src="' . esc_url( $image[0] ) . '" width="32px" heigth="32px" /></a></li>';
						}
					}
					echo '</ul>';
				}

				break;

			case 'text':
				$label     = '';
				$field_key = $field['key'];
				if ( method_exists( $order, "get_$field_key" ) ) {
					$label = $order->{"get_$field_key"}();
				} else {
					$label = $order->get_meta( sprintf( '_%s', $field['key'] ), true );
				}
				echo esc_html( $label );

				break;

			case 'textarea':
				$label = $order->get_meta( sprintf( '_%s', $field['key'] ), true );
				echo wp_kses_post( $label );

				break;

			default:
				$label     = '';
				$field_key = $field['key'];
				if ( method_exists( $order, "get_$field_key" ) ) {
					$label = $order->{"get_$field_key"}();
				} else {
					$label = $order->get_meta( sprintf( '_%s', $field['key'] ), true );
				}
				echo is_array( $label ) ? implode( ', ', esc_html( $label ) ) : esc_html( $label );
		}
	}

	public function render_column_content( $column, $order ) {
		if ( ! is_object( $order ) ) {
			$order = wc_get_order( $order );
		}

		global $post;
		$fields = Plugin::instance()->billing->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( $column === $field['key'] ) {
					$this->render_column_content_field( $order, $field );
				}
			}
		}
		$fields = Plugin::instance()->shipping->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( $column === $field['key'] ) {
					$this->render_column_content_field( $order, $field );
				}
			}
		}
		$fields = Plugin::instance()->additional->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( $column === $field['key'] ) {
					$this->render_column_content_field( $order, $field );
				}
			}
		}
	}

	public function add_sortable_columns( $columns ) {

		$fields = Plugin::instance()->billing->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['sortable'] ) ) {
					$columns[ $field['key'] ] = sprintf( '_%s', $field['key'] );
				}
			}
		}
		$fields = Plugin::instance()->shipping->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['sortable'] ) ) {
					$columns[ $field['key'] ] = sprintf( '_%s', $field['key'] );
				}
			}
		}
		$fields = Plugin::instance()->additional->get_fields();
		if ( $fields ) {
			foreach ( $fields as $field_id => $field ) {
				if ( ! empty( $field['sortable'] ) ) {
					$columns[ $field['key'] ] = sprintf( '_%s', $field['key'] );
				}
			}
		}

		return $columns;
	}

	public function sortable_columns_prepare_items_query_args( $args ) {

		if ( ! is_admin() ) {
			return $args;
		}

		$fields_billing    = Plugin::instance()->billing->get_fields();
		$fields_shipping   = Plugin::instance()->shipping->get_fields();
		$fields_additional = Plugin::instance()->additional->get_fields();

		$fields = array_merge(
			$fields_billing,
			$fields_shipping,
			$fields_additional
		);

		$keys = array_map(
			function ( $item ) {
			return $item['key'];
			},
			$fields
		);

		$orderby = isset( $args['orderby'] ) ? $args['orderby'] : '';

		if ( in_array( ltrim( $orderby, '_' ), $keys, true ) ) {
			$args['orderby']  = 'meta_value';
			$args['meta_key'] = $orderby; // _shipping_address_index
		}

		return $args;
	}

	// Filter
	// ---------------------------------------------------------------------------

	public function search_custom_fields( $wp ) {

		global $typenow;

		if ( 'shop_order' !== $typenow ) {
			return $wp;
		}

		$wp->query_vars = $this->search_custom_fields_legacy( $wp->query_vars );
	}

	public function search_custom_fields_legacy( $args ) {
		// Get all fields
		$fields_billing    = Plugin::instance()->billing->get_fields();
		$fields_shipping   = Plugin::instance()->shipping->get_fields();
		$fields_additional = Plugin::instance()->additional->get_fields();

		$fields = array_merge(
			$fields_billing,
			$fields_shipping,
			$fields_additional
		);

		// Check for fields that are filterable
		$fields = array_filter(
			$fields,
			function ( $item ) {
			return ! empty( $item['filterable'] );
			}
		);

		// Check for every key if is in the request
		foreach ( $fields as $field ) {
			if ( isset( $_REQUEST[ $field['key'] ] ) ) {

				// If is in the request, add to the query checking the type
				$value = trim( wc_clean( wp_unslash( $_REQUEST[ $field['key'] ] ) ) );

				switch ( $field['type'] ) {

					case 'file':
						$args['meta_query'][] = array(
							array(
								'key'     => sprintf( '_%s', $field['key'] ),
								'value'   => "<> ''",
								'type'    => 'numeric',
								'compare' => '!=',
							),
						);
						break;
					case 'multiselect':
					case 'multicheckbox':
						$args['meta_query'][] = array(
							array(
								'key'     => sprintf( '_%s', $field['key'] ),
								'value'   => $value,
								'compare' => 'LIKE',
							),
						);
						break;
					case 'checkbox':
						$args['meta_query'][] = array(
							'relation' => 'OR',
							array(
								'key'     => sprintf( '_%s', $field['key'] ),
								'value'   => 'Yes',
								'compare' => '=',
							),
							array(
								'key'     => sprintf( '_%s', $field['key'] ),
								'value'   => 1,
								'compare' => '=',
							),
							array(
								'key'     => sprintf( '_%s', $field['key'] ),
								'value'   => true,
								'compare' => '=',
							),
						);
						break;
					default:
						$args['meta_query'][] = array(
							array(
								'key'     => sprintf( '_%s', $field['key'] ),
								'value'   => $value,
								'compare' => '=',
							),
						);
				}
			}
		}
		return $args;
	}

	public function search_custom_fields_hpos( $args ) {
		if ( ! current_user_can( 'edit_shop_orders' ) || ! isset( $_REQUEST ) ) {
			return $args;
		}

		// Get all fields
		$fields_billing    = Plugin::instance()->billing->get_fields();
		$fields_shipping   = Plugin::instance()->shipping->get_fields();
		$fields_additional = Plugin::instance()->additional->get_fields();

		$fields = array_merge(
			$fields_billing,
			$fields_shipping,
			$fields_additional
		);

		// Check for fields that are filterable
		$fields = array_filter(
			$fields,
			function ( $item ) {
			return ! empty( $item['filterable'] );
			}
		);

		// Inicializar field_query si no existe
		$args['field_query'] = $args['field_query'] ?? array();

		// Check for every key if is in the request
		foreach ( $fields as $field ) {
			$meta_key = $field['key']; // Usar el key directo sin prefijo

			if ( ! isset( $_REQUEST[ $meta_key ] ) || '' === $_REQUEST[ $meta_key ] ) {
				continue;
			}

			$value = trim( wc_clean( wp_unslash( $_REQUEST[ $field['key'] ] ) ) );

			$is_custom_field = ! method_exists( 'WC_Order', "get_$meta_key" );

			switch ( $field['type'] ) {
				case 'file':
					if ( $is_custom_field ) {
						$args['meta_query'][] = array(
							'key'   => '_' . $meta_key,
							'value' => $value,
						);
					} else {
						$args['field_query'][] = array(
							'field'    => $meta_key,
							'operator' => 'EXISTS',
						);
					}
					break;

				case 'multiselect':
				case 'multicheckbox':
					if ( $is_custom_field ) {
						$args['meta_query'][] = array(
							'key'   => '_' . $meta_key,
							'value' => $value,
						);
					} else {
						$args['field_query'][] = array(
							'field'    => $meta_key,
							'value'    => serialize( $value ),
							'operator' => 'LIKE',
						);
					}
					break;

				case 'checkbox':
					if ( $is_custom_field ) {
						$args['meta_query'][] = array(
							'key'   => '_' . $meta_key,
							'value' => $value,
						);
					} else {
						$args['field_query'][] = array(
							'field'    => $meta_key,
							'value'    => array( 'yes', '1', 'true', 'on', true ),
							'operator' => 'IN',
						);
					}
					break;

				default:
					if ( $is_custom_field ) {
						$args['meta_query'][] = array(
							'key'   => '_' . $meta_key,
							'value' => $value,
						);
					} else {
						$args['field_query'][] = array(
							'field' => $meta_key,
							'value' => $value,
						);
					}
			}
		}

		return $args;
	}

	public function save_order_data( $order_id ) {

		if ( isset( $_POST['woocommerce_meta_nonce'] ) && wp_verify_nonce( wc_clean( wp_unslash( $_POST['woocommerce_meta_nonce'] ) ), 'woocommerce_save_data' ) ) {

			$billing_fields = Plugin::instance()->billing->get_fields();
			$this->save_order_fields( $order_id, $billing_fields, $_POST );

			$shipping_fields = Plugin::instance()->shipping->get_fields();
			$this->save_order_fields( $order_id, $shipping_fields, $_POST );

			$additional_fields = Plugin::instance()->additional->get_fields();
			$this->save_order_fields( $order_id, $additional_fields, $_POST );

		}
	}

	public function save_order_fields( $order_id, $fields, $post_data ) {
		if ( count( $fields ) ) {

			$order = wc_get_order( $order_id );

			foreach ( $fields as $field_id => $field ) {

				$key = sprintf( '_%s', $field['key'] );

				if ( isset( $post_data[ $field['key'] ] ) ) {

					if ( 'textarea' == $field['type'] ) {
						$value = wp_unslash( $post_data[ $field['key'] ] );
						$order->update_meta_data( $key, wp_kses( $value, false ) );
					} else {
						$value = wc_clean( wp_unslash( $post_data[ $field['key'] ] ) );
						$order->update_meta_data( $key, $value );
					}
				}
			}

			$order->save();
		}
	}
}
