<?php

namespace QuadLayers\WOOCCM_PRO\View\Frontend;

use PhpParser\Node\Stmt\Return_;

/**
 * Fields_Display Class
 */
class Fields_Display {

	protected static $_instance;

	public function __construct() {
		add_filter( 'wooccm_checkout_field_filter', array( $this, 'disable_account' ) );
		add_filter( 'wooccm_checkout_field_filter', array( $this, 'disable_checkout' ) );
		add_filter( 'wooccm_checkout_field_filter', array( $this, 'disable_by_cart_subtotal' ), 10 );
		// Remove by product subtype
		add_filter( 'wooccm_checkout_field_filter', array( $this, 'disable_by_product_subtype' ), 50 );

		add_action(
			'woocommerce_before_checkout_form',
			function ( $checkout ) {
				$checkout_fields = $checkout->get_checkout_fields();

				$billing_fields    = isset( $checkout_fields['billing'] ) && is_array( $checkout_fields['billing'] ) ? $checkout_fields['billing'] : array();
				$shipping_fields   = isset( $checkout_fields['shipping'] ) && is_array( $checkout_fields['shipping'] ) ? $checkout_fields['shipping'] : array();
				$additional_fields = isset( $checkout_fields['additional'] ) && is_array( $checkout_fields['additional'] ) ? $checkout_fields['additional'] : array();

				$checkout_fields = array_merge(
					$billing_fields,
					$shipping_fields,
					$additional_fields
				);
				foreach ( $checkout_fields as $key => $field ) {
					if ( isset( $field['user_meta'] ) && false == $field['user_meta'] ) {
						add_filter(
							"default_checkout_$key",
							function ( $value, $input ) {
							return '';
							},
							10,
							2
						);
					}
				}
			},
			10,
			1
		);

		// Action to update user meta on additional checkout fields
		// add_action(
		// 'woocommerce_checkout_update_user_meta',
		// function( $customer_id, $data ) {
		// $customer = WC()->customer;
		// foreach ( $data as $key => $value ) {
		// if ( is_callable( array( $customer, "get_$key" ) ) ) {
		// $value = $customer->{"get_$key"}();
		// } elseif ( $customer->meta_exists( $key ) ) {
		// $value = $customer->get_meta( $key, true );
		// }

		// if ( is_callable( array( $customer, "set_{$key}" ) ) ) {
		// $customer->{"set_{$key}"}( $value );
		// } elseif ( 0 === stripos( $key, 'billing_' ) || 0 === stripos( $key, 'shipping_' ) ) {
		// $customer->update_meta_data( $key, $value );
		// }
		// }
		// },
		// 10,
		// 2
		// );
	}

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function disable_by_product_subtype( $field ) {
		if ( empty( $field['disabled'] ) && ( ! empty( $field['hide_product_subtype'] ) || ! empty( $field['show_product_subtype'] ) ) ) {

			if ( ! isset( WC()->cart ) || ! is_a( WC()->cart, 'WC_Cart' ) ) {
				return $field;
			}

			$cart_contents = WC()->cart->get_cart_contents();
			if ( count( $cart_contents ) ) {

				$hide_product_subtype_array = 'virtual-downloadable' === $field['hide_product_subtype'] ? preg_split( '/-/', $field['hide_product_subtype'] ) : array( $field['hide_product_subtype'] );

				$show_product_subtype_array = 'virtual-downloadable' === $field['show_product_subtype'] ? preg_split( '/-/', $field['show_product_subtype'] ) : array( $field['show_product_subtype'] );

				$apply_conditions_if_more_than_one_product = empty( $field['apply_conditions_if_more_than_one_product'] );

				$products_subtypes = array();
				foreach ( $cart_contents as $key => $values ) {
					$product = wc_get_product( $values['product_id'] );

					$is_downloadable_product = $product->is_downloadable() ? 'downloadable' : '';
					$is_virtual_product      = $product->is_virtual() ? 'virtual' : '';

					if ( $is_downloadable_product && ! in_array( $is_downloadable_product, $products_subtypes ) ) {
						array_push( $products_subtypes, 'downloadable' );
					}
					if ( $is_virtual_product && ! in_array( $is_virtual_product, $products_subtypes ) ) {
						array_push( $products_subtypes, $is_virtual_product );
					}
				}

				// field without more
				// -------------------------------------------------------------------
				if ( $apply_conditions_if_more_than_one_product && count( $cart_contents ) < 2 ) {
					// hide field
					// -----------------------------------------------------------------
					if ( count( $hide_product_subtype_array ) ) {
						if ( array_intersect( $products_subtypes, $hide_product_subtype_array ) ) {
							$field['disabled'] = true;
						}
					}

					// show field
					// -----------------------------------------------------------------
					if ( count( $show_product_subtype_array ) ) {

						if ( ! array_intersect( $products_subtypes, $show_product_subtype_array ) ) {
							$field['disabled'] = true;
						} else {
							$field['disabled'] = false;
						}
					}
				}

				// field with more
				// -------------------------------------------------------------------
				if ( ! $apply_conditions_if_more_than_one_product ) {
					// hide field
					// -------------------------------------------------------------
					if ( count( $hide_product_subtype_array ) ) {
						if ( array_intersect( $products_subtypes, $hide_product_subtype_array ) ) {
							$field['disabled'] = true;
						}
					}

					// show field
					// ---------------------------------------------------------------
					if ( count( $show_product_subtype_array ) ) {

						if ( ! array_intersect( $products_subtypes, $show_product_subtype_array ) ) {
							$field['disabled'] = true;
						} else {
							$field['disabled'] = false;
						}
					}
				}
			}
		}

		return $field;
	}

	public function disable_by_cart_subtotal( $field ) {
		$minimum = (int) $field['show_cart_minimum'];
		$maximum = (int) $field['show_cart_maximun'];

		if ( ! isset( WC()->cart ) || ! is_a( WC()->cart, 'WC_Cart' ) ) {
			return $field;
		}

		if ( $minimum || $maximum ) {

			$subtotal = WC()->cart->get_subtotal();
			if ( $minimum && $maximum ) {
				// between
				$field['disabled'] = ! ( $minimum <= $subtotal && $subtotal <= $maximum );
			} elseif ( $minimum ) {
				// higher than
				$field['disabled'] = ! ( $minimum <= $subtotal );
			} elseif ( '' !== $maximum ) {
				// lower than
				$field['disabled'] = ! ( $subtotal <= $maximum );
			}
		}

		return $field;
	}

	public function disable_account( $field ) {
		if ( ! empty( $field['hide_account'] ) && is_account_page() ) {
			$field['disabled'] = true;
		}

		return $field;
	}

	public function disable_checkout( $field ) {
		if ( ! empty( $field['hide_checkout'] ) && is_checkout() ) {
			$field['disabled'] = true;
		}

		return $field;
	}
}
