<?php

namespace QuadLayers\WOOCCM_PRO\View\Frontend;

use QuadLayers\WOOCCM\Plugin;

/**
 * Fields_Price Class
 */
class Fields_Price {

	protected static $_instance;

	public function __construct() {
		// Add checkout feeds
		add_action( 'woocommerce_cart_calculate_fees', array( $this, 'add_cart_fees' ) );

		// Add price class
		add_filter( 'wooccm_checkout_field_filter', array( $this, 'add_field_class' ) );
	}

	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public function add_checkout_fees( $post_data ) {
		$nonce = isset( $_POST['woocommerce-process-checkout-nonce'] ) ? wc_clean( wp_unslash( $_POST['woocommerce-process-checkout-nonce'] ) ) : ( isset( $post_data['woocommerce-process-checkout-nonce'] ) ? wc_clean( $post_data['woocommerce-process-checkout-nonce'] ) : null );

		if ( wp_verify_nonce( $nonce, 'woocommerce-process_checkout' ) ) {

			$checkout = WC()->checkout->get_checkout_fields();
			if ( count( $checkout ) ) {

				foreach ( $checkout as $field_type => $fields ) {

					foreach ( $fields as $key => $field ) {

						// check if this cart is a recurring cart from Subscriptions
						// if so, check that the add-on is set to be renewable
						// if (is_subscriptions_active() && WC()->cart->start_date && !$field->is_renewable()) {
						// continue;
						// }

						if ( ! empty( $field['add_price_total'] ) ) {
							/*
							$value = isset( $post_data[ $key ] ) ? $post_data[ $key ] : null;
							$value = null === $value && isset( $_POST[ $key ] ) ? $_POST[ $key ] : $value;
							*/

							$value = isset( $post_data[ $key ] ) ? $post_data[ $key ] : null;

							if ( null === $value ) {
								$value = isset( $_POST[ $key ] ) ? wc_clean( wp_unslash( $_POST[ $key ] ) ) : null;
							}

							if ( in_array( $field['type'], array( 'text', 'textarea' ), true ) ) {
								$value = stripslashes( $value );
							}

							if ( null !== $value ) {
								$this->add_field_fee( $key, $field, $value );
							}
						}
					}
				}
			}
		}
	}

	public function add_field_fee( $key, $field = array(), $value = false ) {
		$session_data = WC()->session->wooccm;

		$cost      = 0;
		$taxable   = false;
		$tax_class = '';
		$name      = $field['label'] ? $field['label'] : sprintf( __( 'Fee: %s', 'woocommerce-checkout-manager-pro' ), $key );

		switch ( $field['type'] ) {

			case 'select':
			case 'radio':
				if ( is_array( $field['options'] ) ) {

					foreach ( $field['options'] as $option_key => $option_text ) {

						if ( isset( $field['add_price_total'][ $option_key ] ) && ( $value == html_entity_decode( $option_text ) ) ) {

							$cost = $field['add_price_total'][ $option_key ];
							if ( $cost ) {

								$type    = @$field['add_price_type'][ $option_key ];
								$taxable = @$field['add_price_tax'][ $option_key ];
								// Calculate the percentage if needed.
								if ( 'percent' === $type ) {
									$cost = ( $cost / 100 ) * WC()->cart->cart_contents_total;
								}

								$option_name = $field['options'][ $option_key ];
								$name       .= ' - ' . $option_name;

								WC()->cart->add_fee( $name, $cost, $taxable, $tax_class );
								$session_data['fees'][ $key ] = $cost;
							}
						}
					}
				}

				break;

			case 'multiselect':
			case 'multicheckbox':
				if ( is_array( $field['options'] ) ) {

					foreach ( $field['options'] as $option_key => $option_text ) {

						if ( isset( $field['add_price_total'][ $option_key ] ) && in_array( html_entity_decode( $option_text ), (array) $value, false ) ) {
							$cost = $field['add_price_total'][ $option_key ];
							if ( $cost > 0 ) {

								$type    = @$field['add_price_type'][ $option_key ];
								$taxable = @$field['add_price_tax'][ $option_key ];
								// Calculate the percentage if needed.
								if ( 'percent' === $type ) {
									$cost = ( $cost / 100 ) * WC()->cart->cart_contents_total;
								}

								$option_name   = $field['options'][ $option_key ];
								$complete_name = $name . ' - ' . $option_name;

								WC()->cart->add_fee( $complete_name, $cost, $taxable, $tax_class );
								$session_data['fees'][ $key ] = $cost;

							}
						}
					}
				}

				break;

			default:
				if ( $field['add_price_name'] ) {
					$name = $field['add_price_name'];
				}
				$type    = $field['add_price_type'];
				$taxable = $field['add_price_tax'];

				$cost = $field['add_price_total'];
				if ( $value && $cost > 0 ) {

					if ( 'percent' === $type ) {
						$cost = ( $cost / 100 ) * WC()->cart->cart_contents_total;
					}

					WC()->cart->add_fee( $name, $cost, $taxable, $tax_class );
					$session_data['fees'][ $key ] = $cost;
				}
				break;
		}

		WC()->session->wooccm = $session_data;
	}

	public function add_cart_fees( $cart ) {
		// phpcs:ignore WordPress.Security.NonceVerification.Missing
		if ( ! $_POST || ( is_admin() && ! is_ajax() ) ) {
			return;
		}

		if ( isset( $_POST['post_data'] ) ) {
			// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
			parse_str( wp_unslash( $_POST['post_data'] ), $post_data );
		}

		if ( empty( $post_data ) ) {
			$post_data = array();
		}

		$nonce = isset( $_POST['woocommerce-process-checkout-nonce'] ) ? wc_clean( wp_unslash( $_POST['woocommerce-process-checkout-nonce'] ) ) : ( isset( $post_data['woocommerce-process-checkout-nonce'] ) ? wc_clean( $post_data['woocommerce-process-checkout-nonce'] ) : null );

		// Check fields conditionals.
		$billing_fields    = Plugin::instance()->billing->get_fields();
		$shipping_fields   = Plugin::instance()->shipping->get_fields();
		$additional_fields = Plugin::instance()->additional->get_fields();

		$fields = array_merge( $billing_fields, $shipping_fields, $additional_fields );

		foreach ( $post_data as $key => $value ) {
			if ( ! isset( $fields[ $key ]['conditional'] ) ) {
				continue;
			}

			$conditional_parent_key   = $fields[ $key ]['conditional_parent_key'];
			$conditional_parent_value = $fields[ $key ]['conditional_parent_value'];

			if ( ! $conditional_parent_key ) {
				continue;
			}

			// Check if the key of the conditional parent is present in the form data.
			$is_parent_key_set = isset( $post_data[ $conditional_parent_key ] );

			// If the key of the conditional parent is not set, remove the current key from the form data.
			if ( ! $is_parent_key_set ) {
				unset( $post_data[ $key ] );
			} elseif ( is_array( $post_data[ $conditional_parent_key ] ) ) {
				// If the value of the conditional parent is not in the array values, remove the current key.
				if ( ! in_array( $conditional_parent_value, $post_data[ $conditional_parent_key ], true ) ) {
					unset( $post_data[ $key ] );
				}
			} else {
				// If the value of the conditional parent does not match, remove the current key.
				if ( $post_data[ $conditional_parent_key ] !== $conditional_parent_value ) {
					unset( $post_data[ $key ] );
				}
			}
		}

		if ( wp_verify_nonce( $nonce, 'woocommerce-process_checkout' ) ) {
			$this->add_checkout_fees( $post_data );
		}
	}

	public function add_field_class( $field ) {
		if ( ! empty( $field['add_price_total'] ) && count( array_filter( (array) $field['add_price_total'] ) ) ) {
			$field['class'][] = 'wooccm-add-price';
		}

		return $field;
	}
}
